
    #	Merge multiple transit output files, discarding duplicate
    #	records due to restarts resulting from hiccups.
    
    #	Input:	../Integration_Results/*ward_fine_*.txt
    #	    	    These are the output of transits.c, fed the
    #	    	    stepwise integrator output by the FIFO.  The
    #	    	    names of these files matter!  "forward_fine?.txt"
    #	    	    file are the result of the forward integration
    #	    	    and "backward_fine*.txt" that of the backward
    #	    	    integration--the names are used to distinguish
    #	    	    them and sort them in the right order.  Further,
    #	    	    each file has ac name suffix which must sort them
    #	    	    in the order they were produced.  We use a single
    #	    	    digit for this job, but anything that sorts OK
    #	    	    will work.
    #
    #	Output: all_fine.txt.gz
    #	    	    The file is gzipped on the fly to avoid large file
    #	    	    problems or disc space exhaustion.
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/
    
    use strict;
    
    my $l;
    my $lastjd = -1E30;
    my $lastbary = -1E30;
    
    my ($minjd, $maxjd) = (-43934252.5, 47376737.5);
    my ($dupbary, $duptrans, $mintrim, $maxtrim);
    
    #	Obtain list of transit output files.  Sort them so
    #	backward integration files appear first in reverse order,
    #	followed by forward integration files in ascending
    #	order.
    
    my $intdir = "../Integration_Results";
    opendir(CD, $intdir);   
    my @files = grep(/.*ward_fine.*\.txt/, readdir(CD));
    closedir(CD);

    @files = sort(chronologically @files);
    
    open(FO, "| gzip >all_fine.txt.gz") || die "Cannot open pipe to gzip output file";
    
    #	Read each transit file.  Backward integration files are
    #	read from a pipe from "tac", which reverses the records in
    #	the file so they're in ascending order by time.
    
    my $f;
    
    foreach $f (@files) {
    	print(STDERR "$f\n");
    	my $dfn = "$intdir/$f";
	if ($f =~ m/backward/) {
	    open(FI, "tac $dfn |") || die "Cannot open pipe to tac for $dfn";
    	} else {
	    open(FI, "<$dfn") || die "Cannot open $dfn";
	}
	
	while ($l = <FI>) {
    	    if ($l =~ m/([\d\-]+\.\d+)/) {
		my $jd = $1;

    	    	if (($jd >= $minjd) && ($jd < $maxjd)) {
    	    	    if ($l =~ m/^Barycentre/) {
    			if ($jd > $lastbary) {
	    		    $lastbary = $jd;
			} else {
#		    	    print(STDERR "Dropped duplicate barycentre item (lastjd = $lastjd)\n    $l");
    	    	    	    $dupbary++;
    	    	    	    next;
			}
		    } else {
    			if ($jd >= $lastjd) {
	    		    $lastjd = $jd;
			} else {
#		    	    print(STDERR "Dropped duplicate transit item (lastjd = $lastjd)\n    $l");
    	    	    	    $duptrans++;
    	    	    	    next;
			}
		    }
		    
		    print(FO $l);
    	    	} else {
		    if ($jd >= $minjd) {
		    	$maxtrim++;
		    } else {
		    	$mintrim++;
		    }
		}
    	    } else {
		die("Cannot parse:\n    $l");
    	    }
	}
	
	close(FI);
    }
    close(FO);
    
    print(STDERR "Trimmed $mintrim before, $maxtrim after specified time range.\n");
    print(STDERR "Elided $dupbary duplicate barycentre, $duptrans duplicate transit records.\n");
	
    
    sub chronologically {
    	if (($a =~ m/backward/) && ($b =~ m/backward/)) {
	    return $b cmp $a;
	}
	return $a cmp $b;
    }
